cell = {}

local think = 0
local thinktime = 4
local rushtimer = 5000

local font = love.graphics.newFont(20)
local message = ''
local message_a = 1
local message_y = 10
local messages = {
  'Easy, right?',
  'Any second now...',
  "It's stuffy in there.",
  'Having fun in the only level?',
  'Frantically standing still?',
  "Let's play forever!",
  "Esc means Pause, because you can't escape.",
  "Pressing only 4 keys isn't stressful.",
  'Almost had you. :(',
  "Who's managing these lasers!? ;)",
  "And, breathe."
}

local num = 0 

function cell.initialize(size, msg)
  cell.ready_to_initialize = false
  thinktime = 3
  think = 0
  rushtimer = 5

  num = size > 10 and size or 10 -- Size of the Box
  cell.walls = {
    ['up']    = {},
    ['down']  = {},
    ['left']  = {},
    ['right'] = {}
  }

  local x = (love.graphics.getWidth() / 2) - ((num * wall_size) / 2)
  local y = (love.graphics.getHeight() / 2) - ((num * wall_size) / 2)

  message = msg or messages[love.math.random(#messages)]
  message_a = 1 -- Alpha (It slowly disappears)
  message_y = y - font:getHeight(message) - 15 -- It slowly goes up. 
 
  for i = 1, num - 1 do
    table.insert(cell.walls['up'], newWall(x + (i * wall_size), y, x + (i * wall_size), y + wall_size, wall_size, wall_size * (num - 1)))
    table.insert(cell.walls['down'], newWall(x + (i * wall_size), y + (wall_size * num), x + (i * wall_size), y + wall_size, wall_size, wall_size * (num - 1)))
    
    table.insert(cell.walls['left'], newWall(x, y + (i * wall_size), x + wall_size, y + (i * wall_size), wall_size * (num - 1), wall_size))
    table.insert(cell.walls['right'], newWall(x + (wall_size * num), y + (i * wall_size), x + wall_size, y + (i * wall_size), wall_size * (num - 1), wall_size))
  end
  
  cell.lasers = {}
  
  return cell.walls['down'][1].y + wall_size
end

function cell.update(dt)
  -- Message
  message_a = message_a > 0 and message_a - (dt/3) or 0
  message_y = message_a > 0 and message_y - 10 * dt or  0

  -- Cell Thinktime
  think = think + dt
  if think >= thinktime and rushtimer > 0 then
    think = 0
    thinktime = thinktime > 0.4 and thinktime - 0.2 or 0.4

    -- Choose a wall to charge up
    repeat -- This loop makes sure it doesn't charge up what's ALREADY been charging up
      ready_to_fire = true
      local directions = {'up', 'down', 'left', 'right'}
      local ri = love.math.random(4)
      local rwall = love.math.random(#cell.walls[directions[ri]])
      
      if not cell.walls[directions[ri]][rwall].firing then
        cell.walls[directions[ri]][rwall].charge_up()
      else
        ready_to_fire = false
      end
    until ready_to_fire
  end

  -- Walls
  walls_firing = false
  for ci, _ in pairs(cell.walls) do
    for _, v in pairs(cell.walls[ci]) do
      if v.firing then
        walls_firing = true
        v.firetime = v.firetime - dt
        if v.firetime <= 0 then
          v.fire_laser()
          table.insert(cell.lasers, v.newLaser())
        end
      end
    end
  end

  -- Lasers
  local laser_killer = {}
  for i, v in ipairs(cell.lasers) do
    if v.life <= 0 then table.insert(laser_killer, i) end
    v.update(dt)
  end
  for i = #laser_killer, 1, -1 do
    table.remove(cell.lasers, laser_killer[i])
  end

  if thinktime == 0.4 then
    rushtimer = rushtimer - dt
    -- Let's wait until the last laser has been fired
    if rushtimer <= 0 and #cell.lasers < 1 and not walls_firing then
      cell.ready_to_initialize = true
    end
  end
end
function cell.draw()
  -- Message
  love.graphics.setColor(1,1,1,message_a)
  love.graphics.printf(message, 0, message_y, love.graphics.getWidth(), 'center')
  
  -- Walls
  for ci, _ in pairs(cell.walls) do
    for _, v in pairs(cell.walls[ci]) do
      v.draw()
    end
  end
  
  -- Lasers
  for _, v in pairs(cell.lasers) do
    love.graphics.setColor(1,0,0,v.life)
    v.draw()
  end
end